% Example 2.6 - Linear Discriminant Analysis
% This illustrates a way to reduce the dimensionality to 1-D
% when we have data, where each observation is tagged as
% belonging to one of two classes

% Exploratory Data Analysis with MATLAB, 2nd Edition
% Wendy L. and Angel R. Martinez and Jeff Solka

% We use the function include the mvnrnd function to generate the
% bivariate normal random variables for each class.
% Similar functionality is available in the free Computational
% Statistics Toolbox.

n1 = 100;
n2 = 100;
cov1 = eye(2);
cov2 = [1 .9; .9 1];
dat1 = mvnrnd([-2 2],cov1,n1);
dat2 = mvnrnd([2 -2],cov2,n2);
plot(dat1(:,1),dat1(:,2),'x',...
    dat2(:,1),dat2(:,2),'o')

% Now we estimate the optimal linear mapping according to the LDA
% methodology. The first step is to get the within-class scatter matrix.

% Calculate the scatter matrices, using the fact that they are proportional
% to the sample covariance matrices.
scat1 = (n1-1)*cov(dat1);
scat2 = (n2-1)*cov(dat2);
% Now we compute the within-class scatter matrix.
Sw = scat1 + scat2;

% Next, we need the means for each class in the 
% original space.
mu1 = mean(dat1);
mu2 = mean(dat2);

% The next steps calculate the LDA vector w. 
% Note that we need to take the transpose of
% the means, since they need to be column 
% vectors.
w = inv(Sw)*(mu1' - mu2');
% Normalize the vector w.
w = w/norm(w);

% We can now calculate the projected data.
pdat1 = w'*dat1';
pdat2 = w'*dat2';

% The data now reside in one dimension. So, we will
% construct a kernel density estimate to help us visualize
% it. We also show the data points on the horizontal axis.
% Note that the ksdensity function is in the Statistics Toolbox.
% A similar function is available in the free Computational 
% Statistics Toolbox.

[ksd1,x1] = ksdensity(pdat1);
[ksd2,x2] = ksdensity(pdat2);
plot(pdat1,zeros(1,100),'x',pdat2,zeros(1,100),'o')
hold on
plot(x1,ksd1,x2,ksd2)
axis([-5 8 0 1.5])


